/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Panic Button.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Dependencies:
// - utils.js
//


// Namespace inspired by browser overlay code for Minimize To Tray
if (! ('extensions' in window)) {
  window.extensions = new Array();
}

if (! ('aecreations' in window.extensions)) {
  window.extensions.aecreations = new Array();
}

if (! ('panicbutton' in window.extensions.aecreations)) {
  window.extensions.aecreations.panicbutton = {};
}
else {
  throw new Error("panicbutton object already defined");
}


window.extensions.aecreations.panicbutton = {
  _utils: null,
  _strBundle: null,
  _browserSession: null,
  _toolbarIconCls: [],
  _osEnv: null,

  _cssClass: {
    // The code for the _cssClass object is adapted from "JavaScript: The
    // Definitive Guide" 5/e by David Flanagan (O'Reilly, 2006, pp. 381-382)
    is: function (aElt, aCls) {
      if (typeof aElt == "string") aElt = document.getElementById(aElt);

      var classes = aElt.className;
      if (! classes) return false;
      if (classes == aCls) return true;

      return aElt.className.search("\\b" + aCls + "\\b") != -1;
    },

    add: function (aElt, aCls) {
      if (typeof aElt == "string") aElt = document.getElementById(aElt);
      if (this.is(aElt, aCls)) return;
      if (aElt.className) aCls = " " + aCls;
      aElt.className += aCls;
    },

    remove: function (aElt, aCls) {
      if (typeof aElt == "string") aElt = document.getElementById(aElt);
      aElt.className = aElt.className.replace(new RegExp("\\b" + aCls + "\\b\\s*", "g"), "");
    }
  },


  handleEvent: function (aEvent)
  {
    var that = window.extensions.aecreations.panicbutton;

    if (aEvent.type == "load") {
      that.init();
    }
    else if (aEvent.type == "unload") {
      window.removeEventListener("load",   that, false);
      window.removeEventListener("unload", that, false);      
    }
  },


  init: function ()
  {
    this._initToolbarIconClasses();
    this._utils = new aePanicButtonUtils();
    this._strBundle = document.getElementById("ae-panicbutton-strings");
    try {
      this._browserSession = Components.classes['aecreations@mozdev.org/panicbutton/browser-session;1'].getService(Components.interfaces.nsIBrowserSession);
    }
    catch (e) {
      alert(e);
    }

    this._osEnv = this._utils.getOS();
    this.applyUserPrefs();

    this._utils.log("Panic Button OS environment: " + this._utils.getOS());
  },


  _initToolbarIconClasses: function ()
  {
    this._toolbarIconCls[0]  = "ae-panicbutton-default";
    this._toolbarIconCls[1]  = "ae-panicbutton-exclamation-in-ball";
    this._toolbarIconCls[2]  = "ae-panicbutton-quit";
    this._toolbarIconCls[3]  = "ae-panicbutton-exit-door";
    this._toolbarIconCls[4]  = "ae-panicbutton-window-minimize";
    this._toolbarIconCls[5]  = "ae-panicbutton-window-with-exclamation";
    this._toolbarIconCls[6]  = "ae-panicbutton-window-with-exclamation-ball";
    this._toolbarIconCls[7]  = "ae-panicbutton-window-with-cross";
    this._toolbarIconCls[8]  = "ae-panicbutton-window-with-check";
    this._toolbarIconCls[9]  = "ae-panicbutton-plain-window";
    this._toolbarIconCls[10] = "ae-panicbutton-dotted-window";
    this._toolbarIconCls[11] = "ae-panicbutton-window-with-globe";
    this._toolbarIconCls[12] = "ae-panicbutton-web-page";
    this._toolbarIconCls[13] = "ae-panicbutton-web-page-with-globe";
    this._toolbarIconCls[14] = "ae-panicbutton-web-document";
    this._toolbarIconCls[15] = "ae-panicbutton-smiley";
    this._toolbarIconCls[16] = "ae-panicbutton-picture";
    this._toolbarIconCls[17] = "ae-panicbutton-desktop";
    this._toolbarIconCls[18] = "ae-panicbutton-computer";
    this._toolbarIconCls[19] = "ae-panicbutton-letter-a";
  },


  applyUserPrefs: function ()
  {
    // Enable or disable F9 key
    var keyEnabled = this._utils.getPref("panicbutton.enable_function_key");
    var keysetElt = document.getElementById("mainKeyset");
    var panicButtonKeyElt, macPanicButtonKeyElt;

    for (var i = 0; i < keysetElt.childNodes.length; i++) {
      var child = keysetElt.childNodes[i];
      if (child.id == "ae_key_panicbutton") {
	panicButtonKeyElt = child;
      }
      else if (child.id == "ae_key_mac_panicbutton") {
	macPanicButtonKeyElt = child;
      }
    }

    if (keyEnabled) {
      // The following code works, but the user must still either restart the
      // app or open a new window for the change to take effect.
      if (! panicButtonKeyElt) {
	panicButtonKeyElt = document.createElement("key");
	panicButtonKeyElt.id = "ae_key_panicbutton";
	panicButtonKeyElt.setAttribute("keycode", "VK_F9");
	panicButtonKeyElt.setAttribute("command", "ae_cmd_panicbutton");
	keysetElt.appendChild(panicButtonKeyElt);
      }
      if (! macPanicButtonKeyElt) {
	macPanicButtonKeyElt = document.createElement("key");
	macPanicButtonKeyElt.id = "ae_key_mac_panicbutton";
	macPanicButtonKeyElt.setAttribute("keycode", "VK_F9");
	macPanicButtonKeyElt.setAttribute("modifiers", "meta");
	macPanicButtonKeyElt.setAttribute("command", "ae_cmd_panicbutton");
	keysetElt.appendChild(macPanicButtonKeyElt);     
      }
    }
    else {
      if (panicButtonKeyElt) {
	keysetElt.removeChild(panicButtonKeyElt);
      }
      if (macPanicButtonKeyElt) {
	keysetElt.removeChild(macPanicButtonKeyElt);
      }
    }

    var toolbarBtnElt = document.getElementById("ae-panicbutton-toolbarbutton");
    
    if (toolbarBtnElt) {
      // Set Panic Button toolbar button label
      var label = this._utils.getPref("panicbutton.toolbarbutton.label");
      if (label == "") {
	toolbarBtnElt.label = this._strBundle.getString('panicbutton.defaultLabel');
      }
      else {
	toolbarBtnElt.label = label;
      }

      // Set Panic Button toolbar button image
      var oldCls = this._getOldToolbarButtonClass(toolbarBtnElt);
      var customImgURL = this._utils.getPref("panicbutton.toolbarbutton.custom_icon_url");

      if (customImgURL) {
	// URLs other than file:// are not acceptable - reset the pref and set
	// the toolbar button to the built-in icon that was previously selected
	if (customImgURL.search(/^file:\/\//) == -1) {
	  this._utils.setPref("panicbutton.toolbarbutton.custom_icon_url","");
	  customImgURL = "";
	}
	else {
	  this._cssClass.remove(toolbarBtnElt, oldCls);
	  this._cssClass.add(toolbarBtnElt, "ae-panicbutton-custom-icon");
	  toolbarBtnElt.image = customImgURL;
	}
      }
      else {
	toolbarBtnElt.image = null;
      }

      if (! customImgURL) {
	var iconIdx = this._utils.getPref("panicbutton.toolbarbutton.icon");
	this._cssClass.remove(toolbarBtnElt, oldCls);
	this._cssClass.add(toolbarBtnElt, this._toolbarIconCls[iconIdx]);
      }
    }
  },


  _getOldToolbarButtonClass: function (aToolbarButtonElt)
  {
    var allClassNames = aToolbarButtonElt.className;
    var rv = allClassNames.replace(/toolbarbutton\-1 /, "");
    return rv;
  },


  doPanicAction: function ()
  {
    var action = this._utils.getPref("panicbutton.action", 0);

    if (action == this._utils.PANICBUTTON_ACTION_HIDE) {
      this._closeAll(true);
    }
    else if (action == this._utils.PANICBUTTON_ACTION_MINIMIZE) {
      this._minimizeAll();
    }
    else if (action == this._utils.PANICBUTTON_ACTION_QUIT) {
      this._closeAll(false);
    }
  },


  _closeAll: function (aSaveSession)
  {
    if (aSaveSession) {
      var ss = Components.classes["@mozilla.org/browser/sessionstore;1"]
                         .getService(Components.interfaces.nsISessionStore);
      var state = ss.getBrowserState();
      this._browserSession.data = state;
    }

    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
    // Close any open dialog boxes
    // **NOTE: The enumerator returned by nsIWindowMediator.getEnumerator()
    // will exclude system native file-picker dialogs.
    var dlgEnum = wm.getEnumerator("");

    while (dlgEnum.hasMoreElements()) {
      var dlg = dlgEnum.getNext();
      var docElt = dlg.document.documentElement;
      if (docElt.tagName == "dialog" || docElt.tagName == "prefwindow"
	  || docElt.tagName == "wizard") {
	docElt.cancelDialog();
      }
    }

    var wndEnum = wm.getEnumerator("");

    // Need to do this to make it work on Firefox 3
    window.setTimeout(window.extensions.aecreations.panicbutton
		            ._doCloseAllWindows, 1, aSaveSession, wndEnum);
  },

  
  _doCloseAllWindows: function (aSaveSession, aWndEnum) {
    var that = window.extensions.aecreations.panicbutton;
    that._closeAllWindows.apply(that, arguments);
  },


  _closeAllWindows: function (aSaveSession, aWndEnum)
  {
    // Close browser and ancillary app windows
    while (aWndEnum.hasMoreElements()) {
      var wnd = aWndEnum.getNext();
      wnd.close();
    }

    if (aSaveSession) {
      var wndURL = "chrome://panicbutton/content/panicbuttonToolbar.xul";
      var wndFeatures = "chrome,dialog=0,popup";

      if (this._osEnv == "Darwin") {
	// On Mac OS X, OS_TARGET is "Darwin"
	// Workaround strange behaviour with popups on Mac OS X; see bug 19026
	wndURL += "?tbchrome=none";
	wndFeatures = "chrome,dialog=0,resizable=0";
      }
      else if (this._osEnv == "Linux") {
	// Neither the titlebar nor windoid border colours show up properly on
	// Gnome or KDE; see bug 19051
	wndURL += "?tbchrome=override";
      }

      window.open(wndURL, "ae_pbtb", wndFeatures);
    }
    else {
      var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
                                 .getService(Components.interfaces.nsIAppStartup);
      appStartup.quit(appStartup.eForceQuit);
    }
  },

  
  _minimizeAll: function () 
  {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
    var wndEnum = wm.getEnumerator("");  // Get all host app windows

    while (wndEnum.hasMoreElements()) {
      var wnd = wndEnum.getNext();
      wnd = wnd.QueryInterface(Components.interfaces.nsIDOMChromeWindow);
      wnd.minimize();
    }
  }
};


//
// Event handler initialization
//

window.addEventListener("load", window.extensions.aecreations.panicbutton, false);
window.addEventListener("unload", window.extensions.aecreations.panicbutton, false);
