/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Panic Button.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


var gUtils;
var gBrowserSession;
var gStrBundle;


//
// Floating toolbar window management
//

var gWindoid = {
  titlebarElt: null,

  focus: function (aEvent)
  {
    this.titlebarElt.style.backgroundColor = "activecaption";
  },

  blur:  function (aEvent)
  {
    this.titlebarElt.style.backgroundColor = "inactivecaption";
  }
};


var gWindoidCloseBtn = {
  mouseDown: function (aEvent) 
  {
    if (aEvent.which != 1) {
      return;
    }
  },

  mouseUp: function (aEvent) 
  {
    if (aEvent.which == 1) {
      setTimeout("window.close()", 80);
    }
  }
};


//
// DOM convenience function
//

function $(aID)
{
  return document.getElementById(aID);
}


//
// Toolbar functions
//

function load()
{
  gUtils = new aePanicButtonUtils();
  gBrowserSession = Components.classes['aecreations@mozdev.org/panicbutton/browser-session;1'].getService(Components.interfaces.nsIBrowserSession);
  gStrBundle = $('ae-panicbutton-strings');

  gUtils.log("URL of Restore Session windoid: " + window.location + "\nParameter(s): " + window.location.search);

  gWindoid.titlebarElt = $("windoid-title-bar");
  window.addEventListener("focus", function (e) { gWindoid.focus(e) }, false);
  window.addEventListener("blur", function (e) { gWindoid.blur(e) }, false);

  var tbLayoutPref = gUtils.getPref("panicbutton.restorebar.layout");
  setToolbarLayout(tbLayoutPref, true);

  var tbChromeDisabled = window.location.search == "?tbchrome=none";
  var tbChromeOverride = window.location.search == "?tbchrome=override";

  if (tbChromeDisabled) {
    $("ae-panicbutton-toolbar").style.border = "none";
    $("windoid-title").style.display = "none";
    window.sizeToContent();
  }

  if (tbChromeOverride) {
    $("windoid-title-bar").style.backgroundImage = "url('chrome://panicbutton/skin/images/windoidTitleBar.png')";
    $("windoid-border").style.border = "2px solid #808080";
  }

  window.setTimeout("setToolbarPosition();", 1);
}


function setToolbarPosition()
{
  // Workaround to known problem on Linux where XUL attribute persistence of
  // window coordinates doesn't work.
  var coords = gUtils.getPref("panicbutton.restorebar.position").split(",");
  window.moveTo(coords[0], coords[1]);
}


function setToolbarLayout(aLayoutPref, aOnLoad)
{
  if (! aOnLoad) {
    var currPref = gUtils.getPref("panicbutton.restorebar.layout");
    if (currPref == aLayoutPref) {
      return;
    }
  }

  var buttons = $("windoid-content");

  if (aLayoutPref == gUtils.RESTOREBAR_LAYOUT_ICONSONLY) {
    for (var i = 0; i < buttons.childNodes.length; i++) {
      var childNode = buttons.childNodes[i];
      var label = childNode.getAttribute("label");
      childNode.removeAttribute("label");
      if (gUtils.getPref("panicbutton.restorebar.show_tooltips")) {
	childNode.setAttribute("tooltiptext", label);
      }
    }
  }
  else {
    // aLayoutPref == gUtils.RESTOREBAR_LAYOUT_ICONSANDTEXT
    var restoreBtn = buttons.childNodes[0];
    var exitBtn = buttons.childNodes[1];
    restoreBtn.setAttribute("label", gStrBundle.getString("restoreSession"));
    exitBtn.setAttribute("label", gStrBundle.getString("exit"));

    for (var j = 0; j < buttons.childNodes.length; j++) {
      buttons.childNodes[j].removeAttribute("tooltiptext");
    }
  }

  window.sizeToContent();
  gUtils.setPref("panicbutton.restorebar.layout", aLayoutPref);

  var autoReposition = gUtils.getPref("panicbutton.restorebar.auto_reposition",
				      false);

  if (aLayoutPref == gUtils.RESTOREBAR_LAYOUT_ICONSANDTEXT && autoReposition) {
    // Adjust positioning of windoid if it will disappear into the right edge
    // of the screen - needs to work with dual-monitor setups
    var rightEdgeXPos = window.screenX + window.outerWidth;
    var delta = rightEdgeXPos - window.screen.availWidth;
    if (rightEdgeXPos > window.screen.availWidth) {
      var newXPos = window.screenX - delta;
      window.moveTo(newXPos, window.screenY);
    }
  }
}


function restoreSession()
{
  var ss = Components.classes["@mozilla.org/browser/sessionstore;1"]
                     .getService(Components.interfaces.nsISessionStore);

  ss.setBrowserState(gBrowserSession.data);
  gUtils.setPref("panicbutton.restorebar.position",
		 window.screenX + "," +  window.screenY);
  window.close();
}


function quitBrowser()
{
  var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
                             .getService(Components.interfaces.nsIAppStartup);
  appStartup.quit(appStartup.eForceQuit);
}

