/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Panic Button.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */



function aePanicButtonUtils() {}

aePanicButtonUtils.prototype = {
  DEBUG:   false,

  // Panic button actions
  PANICBUTTON_ACTION_HIDE:      0,
  PANICBUTTON_ACTION_MINIMIZE:  1,
  PANICBUTTON_ACTION_QUIT:      2,

  // Restore toolbar layout options
  RESTOREBAR_LAYOUT_ICONSANDTEXT: 0,
  RESTOREBAR_LAYOUT_ICONSONLY:    1
};


aePanicButtonUtils.prototype.alertEx = function (aTitle, aMessage)
{
  var prmpt = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
  prmpt.alert(null, aTitle, aMessage);
};


aePanicButtonUtils.prototype.confirmEx = function (aTitle, aMessage)
{
  var rv;
  var prmpt = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
  rv = prmpt.confirm(null, aTitle, aMessage);
  return rv;
};


aePanicButtonUtils.prototype.getPref = function (aPrefName, aDefaultValue)
{
  var pref = Components.classes["@mozilla.org/preferences-service;1"]
                       .getService(Components.interfaces.nsIPrefBranch);
  try {
    var prefType = pref.getPrefType(aPrefName);
  } catch (e) {
    this.log(e);
  }
  var rv;
  switch (prefType) {
  case pref.PREF_STRING:
    rv = pref.getCharPref(aPrefName);
    break;

  case pref.PREF_INT:
    rv = pref.getIntPref(aPrefName);
    break;

  case pref.PREF_BOOL:
    rv = pref.getBoolPref(aPrefName);
    break;

  default:
    break;
  }

  if (rv === undefined) {
    rv = aDefaultValue;
  }
  
  return rv;
};

  
aePanicButtonUtils.prototype.setPref = function (aPrefName, aPrefValue)
{
  var pref = Components.classes["@mozilla.org/preferences-service;1"]
                       .getService(Components.interfaces.nsIPrefBranch);
  try {
    var prefType = pref.getPrefType(aPrefName);
  } catch (e) {
    this.log(e);
  }

  switch (prefType) {
    case pref.PREF_STRING:
    pref.setCharPref(aPrefName, aPrefValue);
    break;

    case pref.PREF_INT:
    pref.setIntPref(aPrefName, aPrefValue);
    break;

    case pref.PREF_BOOL:
    pref.setBoolPref(aPrefName, aPrefValue);
    break;

    default:
    break;
  }
};


aePanicButtonUtils.prototype.resetPref = function (aPrefName)
{
  var pref = Components.classes["@mozilla.org/preferences-service;1"]
                       .getService(Components.interfaces.nsIPrefBranch);
  pref.clearUserPref(aPrefName);
};


aePanicButtonUtils.prototype.getCurrentProfileURL = function ()  
{
  var profileDir = this.getCurrentProfileDir();
  var fph = Components.classes["@mozilla.org/network/protocol;1?name=file"].createInstance(Components.interfaces.nsIFileProtocolHandler);
  return fph.getURLSpecFromFile(profileDir);
};


aePanicButtonUtils.prototype.getCurrentProfileDir = function ()
{
  // Throws an exception if profile directory retrieval failed.
  var dirProp = Components.classes["@mozilla.org/file/directory_service;1"]
                          .getService(Components.interfaces.nsIProperties);
  var profileDir = dirProp.get("ProfD", Components.interfaces.nsIFile);
  if (! profileDir) {
    throw "Failed to retrieve current profile directory path";
  }

  return profileDir;
};


aePanicButtonUtils.prototype.getHostAppName = function ()
{
  var rv;
  var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].createInstance(Components.interfaces.nsIXULAppInfo);
  rv = appInfo.name;

  return rv;
}


aePanicButtonUtils.prototype.getHostAppVersionStr = function ()
{
  var rv;
  var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].createInstance(Components.interfaces.nsIXULAppInfo);
  rv = appInfo.version;
  
  return rv;
};


aePanicButtonUtils.prototype.getOS = function ()
{
  var rv;
  var xulRuntime = Components.classes["@mozilla.org/xre/app-info;1"].createInstance(Components.interfaces.nsIXULRuntime);
  rv = xulRuntime.OS;

  return rv;
};


aePanicButtonUtils.prototype.beep = function () 
{
  var sound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
  sound.beep();
};


aePanicButtonUtils.prototype.log = function (aMessage) 
{
  if (this.DEBUG) {
    var consoleSvc = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
    consoleSvc.logStringMessage(aMessage);
  }
};
